"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = exports.CALL_API = void 0;

var _axios = _interopRequireDefault(require("axios"));

var _parseLinkHeader = _interopRequireDefault(require("parse-link-header"));

var _humps = require("humps");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var API_ROOT = process.env.REACT_APP_ENV === 'dev1' ? 'https://api-dev1.trend.community:30443' : process.env.REACT_APP_ENV === 'web1' ? 'https://api.trend.community' : 'http://localhost:3000';

var callApi = function callApi(url, options) {
  return (0, _axios["default"])(url, options).then(function (response) {
    var paginationLinks = (0, _parseLinkHeader["default"])(response.headers.link);
    return _objectSpread({
      data: (0, _humps.camelizeKeys)(response.data)
    }, paginationLinks && paginationLinks.prev && {
      prev: paginationLinks.prev.prev
    }, paginationLinks && paginationLinks.next && {
      next: paginationLinks.next.next
    });
  })["catch"](function (error) {
    if (error.response) {
      // The request was made and the server responded with a status code
      // that falls out of the range of 2xx
      console.error(error.response.data);
      console.error(error.response.status);
      console.error(error.response.headers);
      return Promise.reject(_objectSpread({}, error.response.data, {
        status: error.response.status
      }));
    } else if (error.request) {
      // The request was made but no response was received
      // `error.request` is an instance of XMLHttpRequest in the browser and an instance of
      // http.ClientRequest in node.js
      console.error(error.request);
      return Promise.reject(error.request);
    } else {
      // Something happened in setting up the request that triggered an Error
      console.error('Error', error.message);
      return Promise.reject(error.message);
    }
  });
}; // Action key that carries API call info interpreted by this Redux middleware.


var CALL_API = 'CALL API'; // A Redux middleware that interprets actions with CALL_API info specified.
// Performs the call and promises when such actions are dispatched.

exports.CALL_API = CALL_API;

var _default = function _default(store) {
  return function (next) {
    return function (action) {
      var callAPI = action[CALL_API];

      if (typeof callAPI === 'undefined') {
        return next(action);
      }

      var actionWith = function actionWith(data) {
        var finalAction = _objectSpread({}, action, data);

        delete finalAction[CALL_API];
        return finalAction;
      };

      var endpoint = callAPI.endpoint,
          query = callAPI.query,
          body = callAPI.body,
          method = callAPI.method,
          types = callAPI.types;

      if (typeof endpoint !== 'string') {
        throw new Error('Specify a string endpoint URL.');
      }

      if (!Array.isArray(types) || types.length !== 3) {
        throw new Error('Expected an array of three action types.');
      }

      if (!types.every(function (type) {
        return typeof type === 'string';
      })) {
        throw new Error('Expected action types to be strings.');
      }

      if (!method) {
        throw new Error('Specify a method for api call.');
      }

      if (['GET', 'POST', 'PUT', 'DELETE'].indexOf(method) === -1) {
        throw new Error('Method must be one of {GET, POST, PUT, DELETE}');
      }

      var _types = _slicedToArray(types, 3),
          requestType = _types[0],
          successType = _types[1],
          failureType = _types[2];

      next(actionWith({
        type: requestType
      }));
      var options = {
        method: method,
        params: query,
        data: body,
        withCredentials: true
      };
      return callApi("".concat(API_ROOT, "/").concat(endpoint), options).then(function (response) {
        return next(actionWith({
          response: response,
          type: successType
        }));
      }, function (error) {
        return next(actionWith({
          error: error,
          type: failureType
        }));
      });
    };
  };
};

exports["default"] = _default;